@@ -1,29 +1,3 @@
-multiupload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^22, lease=lease,
-                                           max_concurrent_transfers=10)
-{
-    src_files <- glob2rx(basename(src))
-    src_dir <- dirname(src)
-    src <- dir(src_dir, pattern=src_files, full.names=TRUE)
-
-    if(length(src) == 0)
-        stop("No files to transfer", call.=FALSE)
-    if(length(src) == 1)
-        return(upload_adls_file(filesystem, src, dest, blocksize=blocksize, lease=lease))
-
-    init_pool(max_concurrent_transfers)
-
-    parallel::clusterExport(.AzureStor$pool,
-        c("filesystem", "dest", "blocksize"),
-        envir=environment())
-    parallel::parLapply(.AzureStor$pool, src, function(f)
-    {
-        dest <- sub("//", "/", file.path(dest, basename(f))) # API too dumb to handle //'s
-        AzureStor::upload_adls_file(filesystem, f, dest, blocksize=blocksize, lease=lease)
-    })
-    invisible(NULL)
-}
-
-
 upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lease=NULL)
 {
     src <- normalize_src(src)
@@ -64,36 +38,8 @@ upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lea
 
     # flush contents
     do_container_op(filesystem, dest,
-        options=list(action="flush", position=sprintf("%.0f", pos)),
-        http_verb="PATCH")
-}
-
-
-multidownload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, overwrite=FALSE,
-                                             max_concurrent_transfers=10)
-{
-    src_dir <- dirname(src)
-    if(src_dir == ".")
-        src_dir <- "/"
-
-    files <- list_adls_files(filesystem, src_dir, info="name")
-    src <- grep(glob2rx(src), files, value=TRUE) # file listing on ADLS includes directory name
-
-    if(length(src) == 0)
-        stop("No files to transfer", call.=FALSE)
-    if(length(src) == 1)
-        return(download_adls_file(filesystem, src, dest, blocksize=blocksize, overwrite=overwrite))
-
-    init_pool(max_concurrent_transfers)
-
-    parallel::clusterExport(.AzureStor$pool,
-        c("filesystem", "dest", "overwrite"),
-        envir=environment())
-    parallel::parLapply(.AzureStor$pool, src, function(f)
-    {
-        dest <- file.path(dest, basename(f))
-        AzureStor::download_adls_file(filesystem, f, dest, blocksize=blocksize, overwrite=overwrite)
-    })
+                    options=list(action="flush", position=sprintf("%.0f", pos)),
+                    http_verb="PATCH")
     invisible(NULL)
 }
 
@@ -112,6 +58,8 @@ download_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, o
     {
         if(!overwrite && file.exists(dest))
             stop("Destination file exists and overwrite is FALSE", call.=FALSE)
+        if(!dir.exists(dirname(dest)))
+            dir.create(dirname(dest), recursive=TRUE)
         dest <- file(dest, "w+b")
         on.exit(close(dest))
     }
@@ -143,5 +91,5 @@ download_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, o
     }
 
     bar$close()
-    if(null_dest) dest else invisible(NULL)
+    if(null_dest) rawConnectionValue(dest) else invisible(NULL)
 }
